// import Vue from "vue";
// import debounce from 'lodash/debounce';
// import { DatasetService } from "../../services/dataset.service";
import DatasetService from "../../services/dataset.service";
import { SolrSettings } from "@/models/solr";  // PENDING USE

import { OpenSettings } from "@/models/solr";
import { Component, Vue, Prop, Emit } from "vue-facing-decorator";
import { Dataset, Suggestion, SearchType } from "@/models/dataset";
import { SOLR_HOST, SOLR_CORE } from "@/constants";

import { OPEN_HOST, OPEN_CORE } from "@/constants"; // PENDING USE

@Component({
    name: "VsInput",
})
export default class VsInput extends Vue {
    // @Prop()
    // private title!: string;

    // Define the placeholder text for the input field
    @Prop({ default: "Search" })
    readonly placeholder!: string;

    private display = ""; // Input display value

    @Prop()
    private propDisplay = "";

    private value!: Suggestion | string;
    private error = "";
    private results: Array<Dataset> = []; // Array to store search results
    private loading = false; // Loading state indicator
    private selectedIndex = -1; // Index of the currently selected suggestion
    // private selectedDisplay = "";
    private solr: SolrSettings = {
        core: SOLR_CORE, //"rdr_data", // SOLR.core;
        host: SOLR_HOST, //"tethys.at",
        // core: "test_data", // SOLR.core;
        // host: "repository.geologie.ac.at",
    };

    private open: OpenSettings = {
        core: OPEN_CORE, //"rdr_data", // SOLR.core;
        host: OPEN_HOST, //"tethys.at",
        // core: "test_data", // SOLR.core;
        // host: "repository.geologie.ac.at",
    };

    // private rdrAPI!: DatasetService;
    itemRefs!: Array<Element>; // Array to store references to suggestion items
    emits = ["filter"]; // Emits filter event

    // Set reference for each item
    setItemRef(el: Element): void {
        this.itemRefs.push(el);
    }

    beforeUpdate(): void {
        this.itemRefs = [];
    }

    mounted(): void {
        // this.rdrAPI = new DatasetService();
    }

    get showResults(): boolean {
        return this.results.length > 0;
    }

    get noResults(): boolean {
        return Array.isArray(this.results) && this.results.length === 0;
    }

    get isLoading(): boolean {
        return this.loading === true;
    }

    get hasError(): boolean {
        return this.error !== null;
    }

    // Computed property to generate suggestions based on search results
    get suggestions(): Suggestion[] {
        // const suggestion = {
        //     titles: new Array<string>(),
        //     authors: new Array<string>(),
        //     subjects: new Array<string>(),
        // };
        const suggestions = new Array<Suggestion>();

        console.log("Display:", this.display);
        console.log("results:", this.results );
        
        // Generate suggestions based on search results
        this.results.forEach((dataset) => {

            let foundAny = false;

            console.log("get suggestions:id", dataset.id);
            console.log("get suggestions:title_output", dataset.title_output);
            console.log("get suggestions:author", dataset.author);
            console.log("get suggestions:subjects", dataset.subjects);

            // const del = dataset.title_output?.toLowerCase();
            if (dataset.title_output.toLowerCase().includes(this.display.toLowerCase())) {
                const title = dataset.title_output;
                // if (!suggestion["titles"].find((value) => value === title)) {
                //     suggestion.titles.push(title);
                // }
                // Check if there is already a suggestion with this title and type
                const hasTitleSuggestion = suggestions.some((suggestion) => suggestion.value === title && suggestion.type == SearchType.Title);
                if (!hasTitleSuggestion) {
                    // If there is no such suggestion, create a new one and add it to the suggestions array
                    const suggestion = new Suggestion(title, SearchType.Title);
                    suggestions.push(suggestion);
                    foundAny = true;
                }
            }
            if (this.find(dataset.author, this.display.toLowerCase()) !== "") {
                const author = this.find(dataset.author, this.display.toLowerCase());
                // Check if there is already a suggestion with this author and type
                const hasAuthorSuggestion = suggestions.some((suggestion) => suggestion.value === author && suggestion.type == SearchType.Author);
                if (!hasAuthorSuggestion) {
                    const suggestion = new Suggestion(author, SearchType.Author);
                    suggestions.push(suggestion);
                    foundAny = true;
                }
            }
            if (this.find(dataset.subjects, this.display.toLowerCase()) != "") {
                const subject = this.find(dataset.subjects, this.display.toLowerCase());
                const hasSubjectSuggestion = suggestions.some((suggestion) => suggestion.value === subject && suggestion.type == SearchType.Subject);
                if (!hasSubjectSuggestion) {
                    const suggestion = new Suggestion(subject, SearchType.Subject);
                    suggestions.push(suggestion);
                    foundAny = true;
                }
            }

            // if (!foundAny) {
            //     const suggestion = new Suggestion(dataset.title_output, SearchType.Fuzzy);
            //     suggestions.push(suggestion);
            // }

        });

        return suggestions;
    }

    /**
     *  Clear all values, results and errors
     **/
    clear(): void {
        this.display = "";
        // this.value = null;
        this.results = [];
        this.error = "";
        // this.$emit("clear");
    }

    /* When the search button is clicked or the search input is changed, it updates the value property of the component with the current value of display, 
    and emits a search-change event with the current value of display as the argument. */
    @Emit("search-change")
    search(): string {
        this.results = [];
        // this.$emit("search", this.display)
        this.value = this.display; //(obj["title_output"]) ? obj["title_output"] : obj.id
        return this.display;
    }

    // Handler for search input change
    searchChanged(): void {
        console.log("Search changed!");
        this.selectedIndex = -1;
        // Let's warn the parent that a change was made
        // this.$emit("input", this.display);
        if (this.display.length >= 2) {
            this.loading = true;
            this.resourceSearch();
        } else {
            this.results = [];
        }
    }

    // Perform the search request
    private resourceSearch() {
        if (!this.display) {
            this.results = [];
            return;
        }
        this.loading = true;
        // this.setEventListener();
        this.request();
    }

    // Make the API request to search for datasets
    private request(): void {
        console.log("request()");
        // DatasetService.searchTerm(this.display, this.solr.core, this.solr.host).subscribe({
        DatasetService.searchTerm(this.display).subscribe({
            next: (res: Dataset[]) => this.dataHandler(res),
            error: (error: string) => this.errorHandler(error),
            complete: () => (this.loading = false),
        });
    }

    // Handle the search results
    private dataHandler(datasets: Dataset[]): void {
        this.results = datasets;
        // console.log(datasets);
        
        // this.$emit("search", this.display);
        // this.loading = false;
    }

    // Handle errors from the search request
    private errorHandler(err: string): void {
        this.error = err;
        // this.loading = false;
    }

    /**
     * Is this item selected?
     * @param {Object}
     * @return {Boolean}
     */
    isSelected(key: number): boolean {
        return key === this.selectedIndex;
    }

    // Handle arrow down key press to navigate suggestions
    onArrowDown(ev: Event): void {
        ev.preventDefault();
        if (this.selectedIndex === -1) {
            this.selectedIndex = 0;
            return;
        }
        this.selectedIndex = this.selectedIndex === this.suggestions.length - 1 ? 0 : this.selectedIndex + 1;
        this.fixScrolling();
    }

    // Scroll the selected suggestion into view
    private fixScrolling() {
        const currentElement = this.itemRefs[this.selectedIndex];
        currentElement.scrollIntoView({
            behavior: "smooth",
            block: "nearest",
            inline: "start",
        });
    }

    // Handle arrow up key press to navigate suggestions
    onArrowUp(ev: Event): void {
        ev.preventDefault();
        if (this.selectedIndex === -1) {
            this.selectedIndex = this.suggestions.length - 1;
            return;
        }
        this.selectedIndex = this.selectedIndex === 0 ? this.suggestions.length - 1 : this.selectedIndex - 1;
        this.fixScrolling();
    }

    // Handle enter key press to select a suggestion
    onEnter(): void {
        if (this.selectedIndex === -1) {
            // this.$emit("nothingSelected", this.display);
            this.display && this.search();
        } else {
            this.select(this.suggestions[this.selectedIndex]);
        }

        // this.$emit("enter", this.display);
    }

    @Emit("search-change")
    private select(obj: Suggestion): Suggestion {
        // if (!obj) {
        //     return;
        // }
        this.value = obj; //(obj["title_output"]) ? obj["title_output"] : obj.id
        this.display = obj.value; // this.formatDisplay(obj)
        // this.selectedDisplay = this.display;

        this.close();
        // this.$emit("update", this.value);
        return this.value;
    }

    // Find a search term in an array
    private find(myarray: Array<string>, searchterm: string): string {
        for (let i = 0, len = myarray.length; i < len; i += 1) {
            if (typeof myarray[i] === "string" && myarray[i].toLowerCase().indexOf(searchterm) !== -1) {
                // print or whatever
                return myarray[i];
            }
        }
        return "";
    }

    /**
     *  Close the results list. If nothing was selected clear the search
     */
    close(): void {
        if (!this.value) {
            this.clear();
        }
        // if (this.selectedDisplay !== this.display && this.value) {
        //     this.display = this.selectedDisplay;
        // }
        this.results = [];
        this.error = "";
        //this.removeEventListener()
        // this.$emit("close");
    }
}
